/*
 *	A Test of Lestrade: Card Source Functions
 *
 *	(c) 2004--2005 Martin Mares <mj@ucw.cz>
 *
 *	This software may be freely distributed and used according to the terms
 *	of the GNU Lesser General Public License. See COPYING for details.
 */

#define LOCAL_DEBUG

#include <lestrade/lib.h>
#include <ucw/fastbuf.h>
#include <lestrade/lestrade.h>
#include <custom/custom.h>
#include <custom/debug.h>

#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

static DIR *dir;

static struct fastbuf *
test_card_read(byte *name)
{
  struct fastbuf *i = bopen(name, O_RDONLY, 65536);
  struct fastbuf *x = fbmem_create(1024);
  bbcopy_slow(i, x, ~0U);
  /* FIXME: Should convert the input stream to the tagged format, see sherlock/gather/format/text.c */
  bclose(i);
  bflush(x);
  struct fastbuf *o = fbmem_clone_read(x);
  bclose(x);
  return o;
}

struct fastbuf *
test_card_enum(struct card_attrs *attrs, void *priv)
{
  byte *dirname = priv;

  if (!dir)
    {
      DBG("Opening directory %s", dirname);
      dir = opendir(dirname);
      if (!dir)
	die("opendir(%s): %m", dirname);
    }

  struct dirent *d;
  while (d = readdir(dir))
    {
      byte name[strlen(dirname) + strlen(d->d_name) + 2];
      sprintf(name, "%s/%s", dirname, d->d_name);

      if (strlen(d->d_name) >= CARD_NAME_LENGTH)
	{
	  DBG("Enum: Skipping %s: name too long", name);
	  continue;
	}

      struct stat st;
      if (stat(name, &st))
	{
	  DBG("Enum: Skipping %s: %m", name);
	  continue;
	}
      if (!S_ISREG(st.st_mode) || !(st.st_mode & S_IROTH))
	{
	  DBG("Enum: Skipping %s: Not a file", name);
	  continue;
	}

      DBG("Enum: Reading %s", name);
      struct fastbuf *fb = test_card_read(name);
      strcpy(attrs->name, d->d_name);
      return fb;
    }

  DBG("Done");
  closedir(dir);
  dir = NULL;
  return NULL;
}

struct fastbuf *
test_card_fetch(struct card_attrs *attrs, void *priv)
{
  byte *dirname = priv;
  byte name[strlen(dirname) + CARD_NAME_LENGTH + 2];
  sprintf(name, "%s/%s", dirname, attrs->name);
  DBG("Fetch: Reading %s", name);
  return test_card_read(name);
}
